/*
 *  traqball 2.2
 *  written by Dirk Weber   
 *  http://www.eleqtriq.com/
 *  See demo at: http://www.eleqtriq.com/wp-content/static/demos/2011/traqball2011
 
 *  Copyright (c) 2011 Dirk Weber (http://www.eleqtriq.com)
 *  Licensed under the MIT (http://www.eleqtriq.com/wp-content/uploads/2010/11/mit-license.txt)
 */

(function(){
    var userAgent   = navigator.userAgent.toLowerCase(),
        canTouch    = "ontouchstart" in window,
        prefix      = cssPref = "",
        requestAnimFrame, cancelAnimFrame;
    
    if(/webkit/gi.test(userAgent)){
        prefix = "-webkit-";
        cssPref = "Webkit";
    } else if(/msie | trident/gi.test(userAgent)){
        prefix = "-ms-";
        cssPref = "ms";
    } else if(/mozilla/gi.test(userAgent)){
        prefix = "-moz-";
        cssPref = "Moz";
    } else if(/opera/gi.test(userAgent)){
        prefix = "-o-";
        cssPref = "O";
    } else{
        prefix = "";
    }
    
    function bindEvent(target, type, callback, _remove){
        // //translate events
        // var evType      = type || "touchend";
        // var mouseEvs    = ["mousedown", "mouseup", "mousemove"];
        // var touchEvs    = ["touchstart", "touchend", "touchmove"];
        // var remove      = _remove || "add";
         
        // evType = canTouch ? evType : mouseEvs[touchEvs.indexOf(type)];
        // target[remove+"EventListener"](evType, callback, false);
    }
    
    function getCoords(eventObj){
        var xTouch,
            yTouch;
        
        if(eventObj.type.indexOf("mouse") > -1){
            xTouch = eventObj.pageX;
            yTouch = eventObj.pageY;
        }else if(eventObj.type.indexOf("touch") > -1){
            //only do stuff if 1 single finger is used:
            if(eventObj.touches.length === 1){
                var touch   = eventObj.touches[0];
                xTouch      = touch.pageX;
                yTouch      = touch.pageY;
            }
        }
        
        return [xTouch, yTouch];
    }
    
    function getStyle(target, prop){
        var style = document.defaultView.getComputedStyle(target, "");
        return style.getPropertyValue(prop);
    }
    
    requestAnimFrame = (function(){
        return  window[cssPref+"RequestAnimationFrame"] || function(callback){ window.setTimeout(callback, 17); };
    })();
        
    cancelAnimFrame = (function(){
        return  window[cssPref+"CancelRequestAnimationFrame"] || clearTimeout;
    })();
    
    // Configurable Properties:
    //  decay
    //  alignmentTime
    //  ...

    function Traqball(config) {
        var self = this;
        var radius;                             // prepare a variable for storing the radius of our virtual trackball
        var stage;                              // the DOM-container of our "rotatable" element
        var axis            = [];               // The rotation-axis
        var mouseDownVect   = [];               // Vector on mousedown
        var mouseMoveVect   = [];               // Vector during mousemove
        var startMatrix     = [];               // Transformation-matrix at the moment of *starting* dragging
        var delta;                              // Amount to rotate per frame on a slide
        var decay;                              // How fast delta decays during a slide
        var impulse, pos, w, h, decr, angle, oldAngle, oldTime, curTime, translation, scale;
        this.box = undefined;                   // The traqball object

        var observers = {};                     // Observers
        var orientingVector = [0, 0, 1];    // Vector that keeps track of the orientation of the traqball
        var upVector = [0, -1, 0];               // Vector that keeps track of the up direction of the traqball
        var normals;                            // Provided by the user if they want to define "faces"
        var upVectors;                          // Provided by the user if they want to define "faces"
        var alignmentTime;                      // The amount of time it takes for the dice to align itself on a spin

        // Disables touching the traqball
        this.disable = function() {
            if(self.box !== undefined){
                bindEvent(self.box, 'touchstart', self.evHandlers[0], "remove");
                bindEvent(document, 'touchmove', self.evHandlers[1], "remove");
                bindEvent(document, 'touchend', self.evHandlers[2], "remove");
            }
        };

        // Enables touching the traqball
        this.activate = function(){
            if(self.box !== undefined){
                bindEvent(self.box, 'touchstart', self.evHandlers[0]);
                bindEvent(document, 'touchmove', self.evHandlers[1], "remove");
                bindEvent(document, 'touchend', self.evHandlers[2], "remove");
            }
        };

        //-----------------------------------------------------------------------------------------
        // Rotation functions
        //-----------------------------------------------------------------------------------------

        // Start of user touch on traqball
        function startrotation(e){  
            if(delta !== 0){ stopSlide(); }
            e.preventDefault();
            
            mouseDownVect   = calcZvector(getCoords(e));
            oldTime         = curTime = new Date().getTime();
            oldAngle        = angle; 
        
            bindEvent(self.box,'touchstart', startrotation, "remove");
            bindEvent(document, 'touchmove', rotate);
            bindEvent(document, 'touchend', finishrotation);            
        }

        // User touch on traqball
        function rotate(e){
            var eCoords = getCoords(e);
            e.preventDefault();
            
            oldTime = curTime;
            oldAngle = angle;
            
            // Calculate the currrent z-component of the 3d-vector on the virtual trackball
            mouseMoveVect = calcZvector(eCoords);
        
            // We already calculated the z-vector-component on mousedown and the z-vector-component during mouse-movement. 
            // We will use them to retrieve the current rotation-axis 
            // (the normal-vector perpendiular to mouseDownVect and mouseMoveVect).
            axis[0] = mouseDownVect[1]*mouseMoveVect[2]-mouseDownVect[2]*mouseMoveVect[1];
            axis[1] = mouseDownVect[2]*mouseMoveVect[0]-mouseDownVect[0]*mouseMoveVect[2];
            axis[2] = mouseDownVect[0]*mouseMoveVect[1]-mouseDownVect[1]*mouseMoveVect[0];
            axis    = normalize(axis);
             
            // Now that we have the normal, we need the angle of the rotation.
            // Easy to find by calculating the angle between mouseDownVect and mouseMoveVect:
            angle = calcAngle(mouseDownVect, mouseMoveVect);
            
            //Only one thing left to do: Update the position of the box by applying a new transform:
            // 2 transforms will be applied: the current rotation 3d and the start-matrix
            setRotation();
                                        
            curTime = new Date().getTime();
        }
    
        // End of user touch on traqball
        function finishrotation(e){
            var stopMatrix;
        
            bindEvent(document, 'touchmove', rotate, "remove");
            bindEvent(document, 'touchend', finishrotation, "remove");
            bindEvent(self.box, 'touchstart', startrotation);
            calcSpeed();
            // Starts a slide if the user flings the traqball
            if( impulse && delta > 0){
                // self.disable();
                // requestAnimFrame(slide);
            } else if (!(isNaN(axis[0]) || isNaN(axis[1]) || isNaN(axis[2]))) {
                stopSlide();
            }
        }

        // Sets the 3d transform with axis angle, and the stored transformation matrix
        function setRotation() {
            $.each(axis, function(index, value) { axis[index] = truncate(value); });
            $.each(startMatrix, function(index, value) { startMatrix[index] = truncate(value); });
            angle = truncate(angle);
            updateTransform();
        }

        // Animates setRotation(). When the animation finishes, triggers the spinDone event
        function animateSetRotation() {
            $(self.box).css(prefix + "transition", prefix + "transform "+alignmentTime+"ms ease");
            setRotation();
            setTimeout(function() { 
                $(self.box).css(prefix + "transition", "");
                self.activate();
                self.notifyObservers("spinDone");
            }, alignmentTime);
        }

        //-----------------------------------------------------------------------------------------
        // Matrix and math functions
        //-----------------------------------------------------------------------------------------

        // Stores the axis and angle into the transformation matrix
        function storeMatrix(){
            // Clean up when finishing rotation. Only thing to do: create a new "initial" matrix for the next rotation.
            // If we don't, the object will flip back to the position at launch every time the user starts dragging.
            // Therefore we must:
            // 1. calculate a matrix from axis and the current angle
            // 2. Create a new startmatrix by combining current startmatrix and stopmatrix to a new matrix. 
            // Matrices can be combined by multiplication, so what are we waiting for?
            stopMatrix  = calcMatrix(axis, angle);
            startMatrix = multiplyMatrix(startMatrix, stopMatrix);
            orientingVector = normalize(multiplyVectorMatrix(orientingVector, stopMatrix).splice(0, 3));
            upVector = normalize(multiplyVectorMatrix(upVector, stopMatrix).splice(0, 3));
            axis = [0, 0, 1];
            angle = 0;
        }

        // Calculate the delta impulse used for sliding
        function calcSpeed(){
            var dw  = angle - oldAngle;
                dt  = curTime - oldTime;
                
            delta  = Math.abs(dw * 17 / dt);
            
            if (isNaN(delta)) {
                delta = 0;
            } else if (delta > 1){
                delta = 1;
            }
        } 

        // This function will calculate a z-component for our 3D-vector from the mouse x and y-coordinates 
        // (the corresponding point on our virtual trackball):
        function calcZvector(coords){
            var x       = coords[0] - pos[0];
            var y       = coords[1] - pos[1];         
            var vector  = [(x/radius-1), (y/radius-1)];
            var z       = 1 - vector[0]*vector[0] - vector[1]*vector[1];
                
             // Make sure that dragging stops when z gets a negative value:
            vector[2]   = z > 0 ? Math.sqrt(z) : 0;
 
            return vector;
        }
            
        // Calculate the angle between 2 vectors.
        function calcAngle(v1, v2){
            var numerator   =   v1[0]*v2[0] + v1[1]*v2[1] + v1[2]*v2[2],
                denominator =   Math.sqrt(v1[0]*v1[0] + v1[1]*v1[1] + v1[2]*v1[2])*
                                Math.sqrt(v2[0]*v2[0] + v2[1]*v2[1] + v2[2]*v2[2]),
                angle       =   Math.acos(numerator/denominator);
            
            return angle;
        }
    
        // Creates a rotation matrix from an axis and angle
        // Note: This rotation matrix is for use with CSS 3D
        // May be different than a regular 3d transform matrix
        function calcMatrix(axis, angle){
            var x       = axis[0],
                y       = axis[1],
                z       = axis[2],
                sin     = Math.sin(angle),
                cos     = Math.cos(angle),
                cosmin  = 1-cos,
                matrix  = [(cos+x*x*cosmin), (y*x*cosmin+z*sin),(z*x*cosmin-y*sin),0,
                          (x*y*cosmin-z*sin), (cos+y*y*cosmin), (z*y*cosmin+x*sin),0,
                          (x*z*cosmin+y*sin), (y*z*cosmin-x*sin), (cos+z*z*cosmin),0,
                          0,0,0,1];
                                          
            return matrix;
        }
    
        //findPos-script by www.quirksmode.org
        function findPos(obj) {
            var curleft = 0,
                curtop  = 0;
        
            if (obj.offsetParent) {
                do {
                    curleft += obj.offsetLeft;
                    curtop += obj.offsetTop;
                } while (obj = obj.offsetParent);
                
                return [curleft,curtop];
            }
        }

        //-----------------------------------------------------------------------------------------
        // Slide and spin functions
        //-----------------------------------------------------------------------------------------

        // Starts a slide manually
        // Picks a random impulse and axis to spin on
        this.spin = function(_delta) {
            rotateToRandomFace();
            delta = _delta || getRandomArbitrary(0.75, 1.00);
            axis = [getRandomArbitraryFudge(-1.0, 1.0, 0.5), getRandomArbitraryFudge(-1.0, 1.0, 0.5), getRandomArbitrary(-1.0, 1.0)];
            var mag = Math.sqrt(axis[0] * axis[0] + axis[1] * axis[1] + axis[2] * axis[2]);
            axis[0] = axis[0] / mag;
            axis[1] = axis[1] / mag;
            axis[2] = axis[2] / mag;
            self.disable();
            requestAnimFrame(slide);
        };
        var currentFace;
        this.getCurrentFace = function(){
            return currentFace;
        }
        // Increments rotation based on a given impulse (delta)
        // The speed of the slide decreases eventually to a halt.
        function slide(){
            angle += delta;
            decr = decay*Math.sqrt(delta);
            delta = delta > 0 ? delta-decr : 0;
            setRotation();
            
            // Upon slowing down enough, snap to the face closest to the origin vector
            if (delta <= 0.01){
                var face = findClosestNormal();
                if (face !== undefined) {
                    showVector(normals[face]);
                    setRotation();
                    showVectorUp(upVectors[face]);
                    animateSetRotation();
                    currentFace = face;
                } else {
                    self.activate();
                    self.notifyObservers("spinDone");
                }
                stopSlide();
            } else {
                requestAnimFrame(slide);
            }
        }

        // Stops the slide (the movement of the traqball from an impulse)
        function stopSlide(){
            cancelAnimFrame(slide);
            storeMatrix();
            oldAngle = angle = 0;
            delta = 0;
        }

        // Rotates to a random face (if normals are defined)
        function rotateToRandomFace() {
            if (!normals || !upVectors) return;
            var face = getRandomInt(0, normals.length - 1);
            showVector(normals[face]);
            setRotation();
        }

        //-----------------------------------------------------------------------------------------
        // Transform functions
        //-----------------------------------------------------------------------------------------

        // Resets rotation, scale, and translation values
        this.reset = function() {
            scale = [1, 1, 1];
            translation = [0, 0, 0];
            showVector([0, 0, 1]);
            setRotation();
        };

        // Sets scale for the traqball
        this.scale = function(amount) {
            scale[0] = amount;
            scale[1] = amount;
            scale[2] = amount;
            updateTransform();
        };

        // Sets translation for the traqball
        this.translate = function(x, y, z) {
            translation[0] = x + "px";
            translation[1] = y + "px";
            translation[2] = z + "px";
            updateTransform();
        };

        // Updates the css transformation of the object with axis angle and transformation matrix
        function updateTransform() {
            $(self.box).css(prefix + "transform", "rotate3d("+axis+","+angle+"rad) translate3d("+translation+") scale3d("+scale+") matrix3d("+startMatrix+")");
        }

        //-----------------------------------------------------------------------------------------
        // Face alignment functions
        //-----------------------------------------------------------------------------------------

        // Rotates to show the face defined by the given normal and up vectors
        this.showFace = function(normal, up) {
            showVector(normal);
            // setRotation();
            showVectorUp(up);
            animateSetRotation();
        };

        // Rotates to align the vector given onto the origin vector
        function showVector(vector) {
            var xformedVector = normalize(multiplyVectorMatrix(vector, startMatrix).splice(0, 3));
            var axisAngle = alignVectors(xformedVector, [0, 0, 1]);
            axis = axisAngle.axis;
            angle = axisAngle.angle;
            storeMatrix();
        }

        // Rotates to align the vector given onto the origin vector
        function showVectorUp(vector) {
            var xformedVector = normalize(multiplyVectorMatrix(vector, startMatrix).splice(0, 3));
            // var axisAngle = alignVectors(xformedVector, [0, -1, 0], [0, 0, 1]);
            var axisAngle = alignVectorsOnZ(xformedVector, [0, -1, 0]);
            axis = axisAngle.axis;
            angle = axisAngle.angle;
            storeMatrix();
        }

        // Rotates the object by vector
        function alignToVector(vector) {
            var axisAngle = alignVectors(orientingVector, vector);
            axis = axisAngle.axis;
            angle = axisAngle.angle;
            storeMatrix();
            animateSetRotation();
        }

        // Finds the normal that the closest to the origin vector
        function findClosestNormal() {
            if (!normals || !upVectors) return;
            var smallestAngle = Math.PI * 2;
            var closestNormal = -1;
            var currentMatrix = multiplyMatrix(startMatrix, calcMatrix(axis, angle));
            for (var i = 0; i < normals.length; i++) {
                var _angle = calcAngle([0, 0, 1], multiplyVectorMatrix(normals[i], currentMatrix));
                if (_angle < smallestAngle) {
                    smallestAngle = _angle;
                    closestNormal = i;
                }
            }
            return closestNormal;
        }

        window.findClosestNormal = findClosestNormal;

        //-----------------------------------------------------------------------------------------
        // Utility functions
        //-----------------------------------------------------------------------------------------

        // Truncates a too long of a decimal number to 3 digit precision
        window.truncate = function(number) {
            return Math.round(number * 1000) / 1000;
        };

        //-----------------------------------------------------------------------------------------
        // Observers
        //-----------------------------------------------------------------------------------------

        // Adds an observer
        this.addObserver = function(eventName, callback) {
            observers[eventName] = observers[eventName] || [];
            observers[eventName].push(callback);
        };

        // Triggers observers
        this.notifyObservers = function(eventName, eventData) {
            if (!observers[eventName]) {
                return;
            }
            var length = observers[eventName].length;
            for (var i = 0; i < length; i++) {
                observers[eventName][i](eventData);
            }
        };

        //-----------------------------------------------------------------------------------------
        // Initialization
        //-----------------------------------------------------------------------------------------
        
        // Why is this necessary?   
        // for(var prop in _config){
        //     this.config[prop] = _config[prop];
        // }
        this.disable();
        stage   = document.getElementById(config.stage) || document.getElementsByTagName("body")[0];
        pos     = findPos(stage);
        angle   = config.angle || 0;
        axis = [0, 0, 1];
        alignmentTime = config.alignmentTime || 1000;
        decay = config.decay || 0.025;
        normals = config.normals;
        upVectors = config.upVectors;
        translation = [0, 0, 0];
        scale = [1, 1, 1];
        impulse = config.impulse || false;
        // impulse = this.config.impulse === false ? false : true; // uhhhhh?
             
        // Let's calculate some basic values from "stage" that are necessary for our virtual trackball
        // 1st: determine the radius of our virtual trackball:
        // Note: whenever the trackball is resized, to maintain proper radius, this should be recalculated
        h = stage.offsetHeight / 2,
        w = stage.offsetWidth / 2;
        
        //take the shortest of both values as radius
        radius = h < w ? h : w;
        
        //We parse viewport. The first block-element we find will be our "victim" and made rotatable
        for(var i = 0, l = stage.childNodes.length; i < l; i++) {
            var child = stage.childNodes[i];
                
            if(child.nodeType === 1){
                this.box = child;
                break;
            }
        }
            
        var perspective = getStyle(stage, prefix+"perspective");
        var pOrigin     = getStyle(stage, prefix+"perspective-origin");
        var bTransform  = getStyle(this.box, prefix+"transform");
                            
        //Let's define the start values. If config contains angle or perspective or vector, use them.
        //If not, look for css3d transforms within the CSS.
        //If this fails, let's use some default values.
            
        if (config.axis || config.angle) {
            // Normalize the initAxis (initAxis = axis of rotation) because "box" will look distorted if normal is too long
            axis = normalize(config.axis) || [1, 0, 0];
            angle = config.angle || 0;
            // Last but not least we calculate a matrix from the axis and the angle.
            // This matrix will store the initial orientation in 3d-space
            startMatrix = calcMatrix(axis, angle);
        } else if (bTransform !== "none"){
            //already css3d transforms on element?              
            startMatrix = bTransform.split(",");
            
            //Under certain circumstances some browsers report 2d Transforms.
            //Translate them to 3d:
            if (/matrix3d/gi.test(startMatrix[0])) {
                startMatrix[0] = startMatrix[0].replace(/(matrix3d\()/g, "");
                startMatrix[15] = startMatrix[15].replace(/\)/g, "");
            } else {
                startMatrix[0] = startMatrix[0].replace(/(matrix\()/g, "");
                startMatrix[5] = startMatrix[5].replace(/\)/g, "");
                startMatrix.splice(2,0,0,0);
                startMatrix.splice(6,0,0,0);
                startMatrix.splice(8,0,0,0,1,0);
                startMatrix.splice(14,0,0,1);
            }
            for (var i = 0, l = startMatrix.length; i < l; i++){
                startMatrix[i] = parseFloat(startMatrix[i]);
            }
        } else {
            axis        = [0,1,0];
            angle       = 0;
            startMatrix = calcMatrix(axis, angle);
        }
        
        if(config.perspective) {
            stage.style[cssPref+"Perspective"] = config.perspective;
        } else if (perspective === "none"){
            stage.style[cssPref+"Perspective"] = "700px";
        }
        
        if(config.perspectiveOrigin){
            stage.style[cssPref+"PerspectiveOrigin"] = config.perspectiveOrigin;
        }
        
        updateTransform();
        bindEvent(this.box, 'touchstart', startrotation);
        this.evHandlers = [startrotation, rotate, finishrotation];

        //-----------------------------------------------------------------------------------------
        // End of Object
        //-----------------------------------------------------------------------------------------
    }

    window.Traqball = Traqball;
})();