/* 
Notes
	currentFace == face that is shown
		5 -> 4
		4 -> 5
		3 -> 6
		2 -> 2
		1 -> 3
		0 -> 1
*/

function Dice(sides, htmlID, currentFace) {

	var self = this;
	var cube;
	var folded = true;
	var faces = [];
	var observers = {};
	var borderSize = 3;
	self.currentFace = typeof currentFace=="number" ? currentFace : 5;
	self.main = null;
	self.normals = [];
	self.upVectors = [];
	self.traqBall = null;

	function init() {
		self.doVectors();
		self.main = $("<section id=" + htmlID + "></section>");
		cube = $("<div class='shape-cube'></div>");
		var innerHTML = "<div class='content'></div>";
		faces[0] = $("<figure class='face1'>" + innerHTML + "</figure>");
		faces[1] = $("<figure class='face2'>" + innerHTML + "</figure>");
		faces[2] = $("<figure class='face3'>" + innerHTML + "</figure>");
		faces[3] = $("<figure class='face4'>" + innerHTML + "</figure>");
		faces[4] = $("<figure class='face5'>" + innerHTML + "</figure>");
		faces[5] = $("<figure class='face6'>" + innerHTML + "</figure>");
		for (var i = 0; i < 6; i++) {
			cube.append(faces[i]);
			self.insertContent(i, i + 1);
		}
		self.main.append(cube);
		self.resize();

		$("#dice-container").append(self.main);
		self.traqBall = new Traqball({
			stage: htmlID,
			perspective: 1000,
			normals: self.normals,
			upVectors: self.upVectors
		});
		self.traqBall.disable();
		
		cube.click(function() {
			notifyObservers("onClick");
		});
	}

	self.removeFromDOM = function() {
		self.main.remove();
	};

	self.hide = function() {
		self.main.hide(200);
	};

	self.show = function() {
		self.main.show(200);
	};

	self.move = function(x, y) {
		self.main.css({position: "absolute", left: x, top: y});
	};

	self.scale = function(size) {
		self.main.width(size)
			.height(size);
		cube.width(size).height(size);
		for(var i = 0; i < 6; i++) {
			faces[i].width(size)
				.height(size)
				.css("line-height", size + "px");
			faces[i].find(".content").width(size)
				.height(size);
		}
		self.setFolded(folded);
	};

	self.changeFaceContent = function(face, content) {
		var _content = content || "";
		if(face < sides) {
			self.insertContent(face, _content);
		}
	};

	self.changeFaceColor = function(face, color) {
		var _color = color || "white";
		if(face < sides) {
			self.changeCSS(face, "background", _color);
		}
	};

	self.changeTextColor = function(face, color) {
		var _color = color || "black";
		$(".letter").css("color", _color);
		if(face < sides) {
			self.changeCSS(face, "color", _color);
		}
	};

	self.changeTextSize = function(face, size) {
		var _size = size || "100%";
		if (face < sides) {
			self.changeCSS(face, "font-size", _size);
		}
	};

	self.isFolded = function() {
		return folded;
	};

	self.fold = function() {
    	folded = true;
		var tx = $(cube).outerWidth() / 2;
		self.doVectors();
		faces[0].css("transform", "translateY(-" + tx + "px) translateZ(0px) rotateX(90deg)"); // Top
		faces[1].css("transform", "translateY(0px) translateZ(-" + tx + "px) rotateY(180deg)");	// Back
		faces[2].css("transform", "translateX(" + tx + "px) translateZ(0px) rotateY(90deg)"); // Right
		faces[3].css("transform", "translateY(" + tx + "px) translateZ(0px) rotateX(-90deg)"); // Bottom
		faces[4].css("transform", "translateX(-" + tx + "px) translateZ(0px) rotateY(-90deg)");	// Left
		faces[5].css("transform", "translateX(0px) translateZ(" + tx + "px) rotateY(0deg)"); // Front
	};

	self.doVectors = function() {
		self.normals[0] = [0, -1, 0];
		self.normals[1] = [0, 0, -1];
		self.normals[2] = [1, 0, 0];
		self.normals[3] = [0, 1, 0];
		self.normals[4] = [-1, 0, 0];
		self.normals[5] = [0, 0, 1];
		self.upVectors[0] = [0, 0, -1];
		self.upVectors[1] = [0, -1, 0];
		self.upVectors[2] = [0, -1, 0];
		self.upVectors[3] = [0, 0, 1];
		self.upVectors[4] = [0, -1, 0];
		self.upVectors[5] = [0, -1, 0];
	};

	self.unfold = function() {
		self.traqBall.reset();
		self.traqBall.showFace(self.normals[5], self.upVectors[5]);
		self.traqBall.disable();
		var size = cube.outerWidth() + borderSize;
		folded = false;
		faces[0].css("transform", "translateY(-" + size + "px) translateZ(-" + size / 2 +"px)"); // Top
		faces[1].css("transform", "translateX(-" + size * 2 +"px) translateZ(-" + size / 2 +"px)");	// Back
		faces[2].css("transform", "translateX(" + size + "px) translateZ(-" + size / 2 +"px)");	// Right
		faces[3].css("transform", "translateY(" + size + "px) translateZ(-" + size / 2 +"px)");	// Bottom
		faces[4].css("transform", "translateX(-" + size + "px) translateZ(-" + size / 2 +"px)"); // Left
		faces[5].css("transform", "translateY(0px) translateZ(-" + size / 2 +"px)"); // Front
	};

	self.setFolded = function(value) {
		if(value) {
			self.fold();
		} else {
			self.unfold();
		}
	};

	self.setFace = function(face){
		self.traqBall.showFace(self.normals[face], self.upVectors[face]);
	};

	self.endSpin = function() {
		self.currentFace = self.traqBall.getCurrentFace();
		self.traqBall.disable();
		var css = {
			"transition-duration": "0s",
			"-webkit-transition-duration": "0s",
		};
		cube.css(css);
	};

	//CUBE
	self.insertContent = function(face, content) {
		faces[face].find(".content")[0].innerHTML = content;
	};

	self.changeCSS = function(face, rule, value) {
		faces[face].find(".content").css(rule, value);
	};

	self.getSize = function() {
		return cube.outerWidth();
	};

	self.resize = function() {
		var size = cube.outerWidth();
		cube.height(size);
		for(var i = 0; i < 6; i++) {
			faces[i].width(size)
				.height(size)
				.css("line-height", size + "px");
			faces[i].find(".content").width(size)
				.height(size);
		}
		self.setFolded(folded);
	};

	self.addObserver = function(eventName, callback) {
		observers[eventName] = observers[eventName] || [];
		observers[eventName].push(callback);
	};

	function notifyObservers(eventName, eventData) {
		if (!observers[eventName]) {
			return;
		}
		var length = observers[eventName].length;
		for (var i = 0; i < length; i++) {
			observers[eventName][i](eventData);
		}
	}

	init();
	

}


