function saveData() {
	var currentFaces = [];
	for(var i = 0; i < dice.length; i++) {
		currentFaces[i] = dice[i].currentFace;
	}
	var sidesContent = [];
	var textTheme = diceThemer.getThemeOption("Letters");
	var customTheme = diceThemer.getThemeOption("Custom");
	for(var i = 0; i < 6; i++) {
		var jsonVal = {
			textContent: textTheme.content[i],
			customContent:  customTheme.content[i],
		};
		sidesContent.push(jsonVal);
	}
	var data = {
		editMenuIsOpen: editMenuIsOpen,
		rgb: currentRgb,
		currentFaces: currentFaces,
		sidesContent: sidesContent,
		currentTheme: theme.type,
		isFolded: dice[0].isFolded(),
	};
	NB.persist.save("data", data);
}

function updateInfoText(){
	var text = $("#info-text");
	if(theme.type === "Traditional" || !dice[0] || dice[0].isFolded()){
		text.css("opacity", 0);
	} else if(theme.type === "Letters") {
		text.text(localizedStrings.text_helper);
		text.css("opacity", 1);
	} else if(theme.type === "Custom") {
		text.text(localizedStrings.custom_helper);
		text.css("opacity", 1);
	}
}


function DiceController() {
	
	//Properties
	var self = this;
	var nbImageSniffer = null;
	self.settingsButton = new SettingsButton("img/gear_icon.png");
	var diceSpin = null;
	var sizeCache = new SizeCache();
	var selectionListener;
	//Init
	function init() {
		currentRgb = [255, 255, 255];
		initColorPicker();

		//Settings Button
		var btn = self.settingsButton;
		btn.animDuration = 150;
		$("#dice-container").prepend(btn.jq);
		btn.back.css("background-color", "#00BFDD");
		btn.img.click(self.onSettingsClick);
		
		//Image Sniffer
		nbImageSniffer = new NBImageSniffer();
		nbImageSniffer.addObserver(onImageSniffed);
		
		//Events
		$("#traditional-theme").click(function() {
			self.setTheme("Traditional");
		});

		$("#letters-theme").click(function() {
			self.setTheme("Letters");
		});

		$("#custom-theme").click(function() {
			self.setTheme("Custom");
		});
		$("#controls-add").click(function() {
			diceController.pushDice();
		});
		$("#controls-remove").click(function() {
			diceController.popDice();
		});
		$("#color").mousedown(function(event){
			event.stopPropagation();
			$("#color-picker-wrap").show();
			self.updatePreview(100);
		});
		$("#container").mousedown(function(){
			$("#color-picker-wrap").hide();
		});
		$("#controls-edit").click(self.onSettingsClick);
		$(window).resize(self.onWindowResize)
			.focusout(self.onFocusout);

		//Load
		if(!loadData() && !self.loadFlash()) {
			dice = [];
			var htmlID = "6" + "sides-" + 0 + "index";
			var newDice = new Dice(6, htmlID);
			newDice.addObserver("onClick", self.onDiceClick);
			dice[0] = newDice;
			self.initialize(false, true, currentRgb, "Traditional");
		}

		//Other Init
		self.updateMinSizes();
		self.updatePlusMinus();
		self.layout();
		updateInfoText();
		for(die in dice){
			dice[die].setFace(dice[die].currentFace);
		}
		self.selectionListener = new SelectionListener(self.onSelectionChange);
		setTimeout(self.layout, 10); //Safe layout for any intermittent layout issues on load, etc.
		
	}

	self.initialize = function(menuIsOpen, isFolded, rgb, themeName) {
		self._setEditMenuIsOpen(menuIsOpen, isFolded);
		currentRgb = rgb;
		self.setDiceRgb(currentRgb);
		colorPicker.setRgb(currentRgb);
		$("#color").css("background-color", ColorPicker.rgbToString(currentRgb));
		self._setTheme(diceThemer.getThemeOption(themeName));
	};

	function initColorPicker() {
		colorPicker = new ColorPicker();
		$("#color-picker-wrap").append(colorPicker.jq);
		colorPicker.jq.attr("id", "color-picker");
		colorPicker.setHexBorder(0.06, '#585858');
		colorPicker.layout({
			wheelWidth : 0.9,
			hexWidth: 0.16,
			wheelThickness: 0.15,
		});
		$("#color-picker-wrap").hide();
		colorPicker.onHexClick = function(picker, newRgb) {
			currentRgb = newRgb.slice();
			diceController.setDiceRgb(newRgb);
			diceController.updatePreview();
			saveData();
		};
		colorPicker.onWheelUp = function(picker, newRgb) {
			saveData();
			diceController.updatePreview(100);
		};
		colorPicker.onDrag = function(picker, newRgb) {
			currentRgb = newRgb.slice();
			diceController.setDiceRgb(newRgb);
		};
		colorPicker.onXClick = function(picker) {
			$("#color-picker-wrap").hide();
			diceController.updatePreview(100);
		};
	}

	//Persist
	function loadData(){
		var stringParams = NB.persist.load("data");
		if(typeof stringParams == "string" && stringParams.length > 0) {
		    //Dice
		    var params = JSON.parse(stringParams);
		    dice = [];
		    for(var i = 0; i < params.currentFaces.length; i++) {
		    	var face = params.currentFaces[i];
		    	dice[i] = new Dice(6, "6" + "sides-" + i + "index", face);
		    	dice[i].addObserver("onClick", self.onDiceClick);
		    	dice[i].setFace(face);
		    }

		    //Themes
		    var textTheme = diceThemer.getThemeOption("Letters");
			var customTheme = diceThemer.getThemeOption("Custom");
		    for(var i = 0; i < 6; i++) {
		    	var content = params.sidesContent[i];
		    	textTheme.content[i] = content.textContent;
		    	customTheme.content[i] = content.customContent;
		    }
		    self.initialize(params.editMenuIsOpen, params.isFolded, params.rgb, params.currentTheme)
		    return true;
		}
		return false;
	}

	self.loadFlash = function() {
		var flashId = NB.persist.load("updateDataFlashId");
		var updateData = NB.persist.load("updateData");

		if (!flashId) {
			return false;
		}
		theme = diceThemer.getThemeOption("Traditional"); //Theme to apply
		var textTheme = diceThemer.getThemeOption("Letters");
		var customTheme = diceThemer.getThemeOption("Custom");
		if(flashId == "Dice-Ian-black") {
			currentRgb = [0, 0, 0];
		} else if((flashId == "Dice-Ian-blue") ||
				(flashId == "Dice-Ian-blue-english") ||
				(flashId == "Dice-Ian-blue-french") ||  
				(flashId == "Dice-Ian-blue-spanish") ||
				(flashId == "Dice-double")) {
			currentRgb = [0, 106, 255];
		} else if(flashId == "Dice-Ian-pink") {
			currentRgb = [194, 46, 195];
		} else if(flashId == "Dice-Ian-Red") {
			currentRgb = [220, 0, 0];
		} else if(flashId == "Dice-Ian-green") {
			currentRgb = [23, 127, 0];
		} else if((flashId == "Dice-Ian-blue-english-numbers") ||
				(flashId == "Dice-Ian-blue-french-numbers") ||
				(flashId == "Dice-Ian-blue-spanish-numbers")) {
			currentRgb = [0, 106, 255];
			theme = textTheme;
			var nums = [1, 3, 2, 6, 5, 4];
			for (var i = 0; i < 6; i++) {
				textTheme.content[i] = getLetterSpan(i + 1, nums[i].toString());
			}
		} else if(flashId == "Dice-Ian-blue-text-vowels") {
			currentRgb = [0, 106, 255];
			theme = textTheme;
			var vowels = ["e", "", "o", "u", "i", "a"] ;
			for (var i = 0; i < 6; i++) {
				textTheme.content[i] = getLetterSpan(i + 1, vowels[i]);
			}
		} else if(flashId == "Dice-Ian-blue-text-constenants") {
			currentRgb = [0, 106, 255];
			theme = textTheme;
			var texts = ["b", "c", "d", "f", "g", "h"];
			for (var i = 0; i < 6; i++) {
				textTheme.content[i] = getLetterSpan(i + 1, texts[i]);
			}
		} else if(flashId == "KeywordDiceBlue") {
			currentRgb = [0, 106, 255];
			theme = textTheme;
			for (var i = 0; i < 6; i++) {
				textTheme.content[i] = getLetterSpan(i + 1, ""); //clear text sides
			}
			try {
				var xml = $.parseXML(updateData);
				for (var i = 0; i < 6; i++) {
					var savedSideNode = xml.getElementsByTagName("SavedSide" + (i + 1).toString())[0];
					var text = savedSideNode.textContent;
					textTheme.content[i] = getLetterSpan(i + 1, text);
				}
			} catch(e) {} //Do Nothing, its fine if we get malformed xml
		} else if(flashId == "KeywordDiceImageBlue") {
			currentRgb = [0, 106, 255];
			theme = customTheme;
			var urls = [];
			urls.length = 6;
			for (var i = 0; i < 6; i++) {
				try {
					var data = NB.persist.load("my_mc" + (i + 1).toString());
					urls[i] = "_nb/" + NB.HOSTGUID + "/" + data.guid;
				} catch(e) {} //Unable to get image
			}
			new FlashImgConverter(urls, self.onFlashConversion);
		}
		dice = [];
		var newDice = new Dice(6, "6sides-" + 0 + "index");
		newDice.addObserver("onClick", self.onDiceClick);
		dice[0] = newDice;
		dice[0].setFace(5);
		self.flashResize();
		self.initialize(false, true, currentRgb, theme.type);
		// self.removeFlashData();
		return true;
	};

	self.removeFlashData = function() {
		NB.persist.remove("updateDataFlashId");
		NB.persist.remove("updateData");
		for (var i = 1; i <= 6; i++) {
			NB.persist.remove("my_mc" + i);
		}
	};

	self.flashResize = function() {
		var host = NB.getHostObject();
		var width = host.width;
		var mins = self.minSizesFor(false);
		if(width < mins.width) {
			width = mins.width;
		}
		self.setAnnSize(width, width);
	};

	self.onFlashConversion = function(converter, newUrls) {
		var customTheme = diceThemer.getThemeOption("Custom");
		try {
			for (var i = 0; i < 6; i++) {
				var url = newUrls[i];
				if(url) {
					customTheme.content[i] = diceThemer.getImageHTML(url);
				}
			}
			updateDiceTheme();
		} catch(e) {} //Couldn't load images
		
	};

	//Color
	self.setDiceRgb = function(rgb) {
    	var color = ColorPicker.rgbToString(rgb);
		$('#color').css({
			backgroundColor: color,
		});
		var textColor = ColorPicker.rgbToHsl(rgb)[2] < 0.35 ? "white" : "black";
		self.updateDotColor(rgb);
		self.updateCustomPicColor(rgb);
		for(var i = 0; i < dice.length; i++) {
			setDiceColors(dice[i], color, textColor);
		}
    };

	self.updateDotColor = function(diceRgb) {
		if(ColorPicker.rgbToHsl(diceRgb)[2] > 0.35) {
			$(".traditional-img-b").show();
			$(".traditional-img-w").hide();
		} else {
			$(".traditional-img-b").hide();
			$(".traditional-img-w").show();
		}
	};

	self.updateCustomPicColor = function(diceRgb) {
		if(ColorPicker.isDarkText(diceRgb)) {
			$(".custom-pic-b").show();
			$(".custom-pic-w").hide();
		} else {
			$(".custom-pic-b").hide();
			$(".custom-pic-w").show();
		}
	};

	function setDiceColors(_dice, color, textColor) {
		for(var i = 0; i < 6; i++) {
			_dice.changeFaceColor(i, color);
			_dice.changeTextColor(i, textColor);
		}
	}

	//Dice
	function scaleAllDice(scale) {
		for(var i = 0; i < dice.length; i++) {
			dice[i].scale(scale);
		}
	}

	self.setDice = function(_dice){
		dice = _dice;
	};

	self.pushDice = function() {
		if(dice.length < 9) {
			var htmlID = 6 + "sides-" + dice.length + "index";
			var die = new Dice(6, htmlID);
			die.addObserver("onClick", self.onDiceClick);
			die.setFace(die.currentFace);
			dice.push(die);
			self.handlePlusMinus();
		}
	};

	self.popDice = function() {
		if(dice.length > 1) {
			var lostDice = dice.pop();
			lostDice.removeFromDOM();
			self.handlePlusMinus();
		}
	};

	self.handlePlusMinus = function() {
		updateDiceTheme();
		self.setDiceRgb(currentRgb);
		self.setFolded(true);
		self.updateTextEnabledDelayed();
		self.layout();
		self.updatePlusMinus();
		saveData();
	};

	//Spin
	function spinAllDice() {
		if(diceSpin === null) {
			diceSpin = new DiceSpin(onSpinEnd);
			diceSpin.start();
			self.settingsButton.setShownAnim(false);
		}
	}

	function onSpinEnd(x) {
		diceSpin = null;
		saveData();
		self.updatePreview(500);
		if(document.hasFocus() || self.selectionListener.isSelected()) {
			self.settingsButton.setShownAnim(true);
		}
	}

	//Fold
	self.setFolded = function(value) {
		if(value) {
			dice[0].fold();
			for(var i = 1; i < dice.length; i++) {
				dice[i].show();
			}
			$("#info-text").hide();
		} else {
			dice[0].unfold();
			for(var i = 1; i < dice.length; i++) {
				dice[i].hide();
			}
			$("#info-text").show();
		}
		self.updateTextEnabledDelayed();
	};

	//Layout
	self.layout = function() {
		//Top level layout function, call this upon resize, or to update
		var width = $("#container").width();
		var top = 0;
		if(editMenuIsOpen) {
			width -= $("#settings-menu").width();
			top = $("#controls-cont").height();
		}
		$("#dice-container").width(width)
			.css({
				top: top,
			});
		var sections = $("#dice-container section");
		sections.height(sections.width());
		self.layoutDice();
		var fontSize = width / 20;
		if(fontSize > 14) {
			fontSize = 14;
		}
		$("#info-text").css("font-size", fontSize + "px");
		self.layoutSettingsButton();
		self.updatePreview(500);
	};

	// There was no elegant way to handle some of the irregular dice layouts
	// so the location values are hard-coded in here
	self.layoutDice = function() {
		var width = $("#dice-container").width();
		var height = $("#dice-container").height();
		var size;
		if(dice[0].isFolded()) {
			switch(dice.length) {
				case 1 :
					size = 0.52 * width;
					dice[0].move((width - size) / 2, (height - size) / 2);
					break;
				case 2 :
					size = width / 3.5;
					var iGap = (Math.min(width, height) - (size * 2)) / 5;
					var gapX = (width - (size * 2) - iGap) / 2;
					var gapY = (height - (size * 2) - iGap) / 2;
					dice[0].move(gapX, size + gapY + iGap);
					dice[1].move(size + gapX + iGap, gapY);
					break;
				case 3 :
					size = width / 4;
					var iGap = (Math.min(width, height) - (size * 2)) / 3;
					var gapX = (width - (size * 2) - iGap) / 2;
					var gapY = (height - (size * 2) - iGap) / 2;
					dice[0].move(gapX, gapY);
					dice[1].move(size + gapX + iGap, gapY);
					dice[2].move(width / 2 - size / 2, size + gapY + iGap);
					break;
				case 4 :
					size = width / 4;
					var iGap = (Math.min(width, height) - (size * 2)) / 3;
					var gapX = (width - (size * 2) - iGap) / 2;
					var gapY = (height - (size * 2) - iGap) / 2;
					dice[0].move(gapX, gapY);
					dice[1].move(size + gapX + iGap, gapY);
					dice[2].move(gapX, size + gapY + iGap);
					dice[3].move(size + gapX + iGap, size + gapY + iGap);
					break;
				case 5 :
					size = width / 5.5;
					var iGap = (Math.min(width, height) - size * 3) / 8;
					var oGapX = (width - (size * 3) - (iGap * 2)) / 2;
					var oGapY = (height - (size * 3) - (iGap * 2)) / 2;
					dice[0].move(oGapX, oGapY);
					dice[1].move(oGapX + 2 * iGap + 2 * size, oGapY);
					dice[2].move(iGap + oGapX + size, iGap + oGapY + size);
					dice[3].move(oGapX, oGapY + 2 * iGap + 2 * size);
					dice[4].move(oGapX + 2 * iGap + 2 * size, oGapY + 2 * iGap + 2 * size);
					break;
				case 6 :
					size = width / 5;
					var iGap = (height - size * 3) / 4;
					var oGapV = (height - (size * 3) - (iGap * 2)) / 2;
					var oGapH = (width - (size * 2) - iGap) / 2;
					dice[0].move(oGapH, oGapV);
					dice[1].move(oGapH + iGap + size, oGapV);
					dice[2].move(oGapH + iGap + size, iGap + oGapV + size);
					dice[3].move(oGapH, oGapV + 2 * iGap + 2 * size);
					dice[4].move(oGapH + iGap + size, oGapV + 2 * iGap + 2 * size);
					dice[5].move(oGapH, oGapV + iGap + size);
					break;
				case 7 :
					size = width / 5;
					var iGap = (Math.min(width, height) - size * 3) / 4;
					var oGapX = (width - (size * 3) - (iGap * 2)) / 2;
					var oGapY = (height - (size * 3) - (iGap * 2)) / 2;
					dice[0].move(oGapX + (iGap + size) / 2, oGapY);
					dice[1].move(oGapX + 3 * (iGap + size) / 2, oGapY);
					dice[2].move(iGap + oGapX + size, iGap + oGapY + size);
					dice[3].move(oGapX + (iGap + size) / 2, oGapY + 2 * iGap + 2 * size);
					dice[4].move(oGapX + 3 * (iGap + size) / 2, oGapY + 2 * iGap + 2 * size);
					dice[5].move(oGapX, oGapY + iGap + size);
					dice[6].move(oGapX + iGap * 2 + size * 2, oGapY + iGap + size);
					break;
				case 8 :
					size = width / 6.5;
					var iGap = (Math.min(width, height) - size * 3) / 5;
					var oGapX = (width - (size * 3) - (iGap * 2)) / 2;
					var oGapY = (height - (size * 3) - (iGap * 2)) / 2;
					dice[0].move(oGapX, oGapY);
					dice[1].move(oGapX + 2 * iGap + 2 * size, oGapY);
					dice[2].move(iGap + oGapX + size, oGapY + (iGap + size) / 2);
					dice[3].move(oGapX, oGapY + 2 * iGap + 2 * size);
					dice[4].move(oGapX + 2 * iGap + 2 * size, oGapY + 2 * iGap + 2 * size);
					dice[5].move(oGapX, oGapY + iGap + size);
					dice[6].move(oGapX + iGap * 2 + size * 2, oGapY + iGap + size);
					dice[7].move(iGap + oGapX + size, oGapY + 3 * (iGap + size) / 2);
					break;
				case 9 :
					size = width / 6.5;
					var iGap = (Math.min(width, height) - size * 3) / 5;
					var oGapX = (width - (size * 3) - (iGap * 2)) / 2;
					var oGapY = (height - (size * 3) - (iGap * 2)) / 2;
					dice[0].move(oGapX, oGapY);
					dice[1].move(oGapX + 2 * iGap + 2 * size, oGapY);
					dice[2].move(iGap + oGapX + size, iGap + oGapY + size);
					dice[3].move(oGapX, oGapY + 2 * iGap + 2 * size);
					dice[4].move(oGapX + 2 * iGap + 2 * size, oGapY + 2 * iGap + 2 * size);
					dice[5].move(oGapX, oGapY + iGap + size);
					dice[6].move(oGapX + iGap * 2 + size * 2, oGapY + iGap + size);
					dice[7].move(iGap + oGapX + size, oGapY);
					dice[8].move(iGap + oGapX + size, oGapY + 2 * (size + iGap));
					break;
			}
		} else {
			size = 0.2 * width;
			dice[0].move(0.5 * width, (width - size) * 0.5 + 10);
		}		
		scaleAllDice(size);
		resizeAllDiceText();
	};

	self.layoutSettingsButton = function() {
		var btn = self.settingsButton;
		
		//Get Die
		var dieIndex = dice.length > 1 ? 1 : 0;
		var die = dice[dieIndex];
		var pos = die.main.position();
		var dieWidth = die.main.width();
		var extraMarg = 0.11 * dieWidth;
		var containerWidth = $("#dice-container").width();

		//Layout
		var width = dieWidth * 0.5;
		if(width < 40) {
			width = 40;
		} else if(width > 60) {
			width = 60;
		}
		var perspective = 1000;
		var cubeEdge = perspective * dieWidth / 2 / (perspective - dieWidth / 2); //distance from center to cube edge
		var x = pos.left + dieWidth / 2 + cubeEdge - width * 0.5 + 2;
		if(x + width > containerWidth) {
			x = containerWidth - width;
		}
		btn.jq.css({
			width: width,
			height: width * 0.5,
			left: x,
			top: pos.top + dieWidth / 2 - cubeEdge,
		});
		var imgWidth = width * 0.4;
		btn.img.css({
			right: 1,
			top: width * 0.05,
			width: imgWidth,
			height: imgWidth,
		});

	};

	self.setSize = function(size) {
		updateDiceTheme();
	};


	self._setEditMenuIsOpen = function(isOpen, folded) {
		editMenuIsOpen = isOpen;
		if(editMenuIsOpen) {
			$("#settings-back").show();
			self.settingsButton.jq.hide();
		} else {
			$("#settings-back").hide();
			self.settingsButton.jq.show();
		}
		self.setFolded(folded);
	};

	self.toggleSettings = function() {
		editMenuIsOpen = !editMenuIsOpen;
		if(editMenuIsOpen) {
			sizeCache.cache();
		}
		if(!editMenuIsOpen) {
			updateDiceTheme(); //Write content to other dice.
		}

		//Update Dimensions
		if(!editMenuIsOpen && sizeCache.isValid()) { //Check Cache
			self.setAnnSize(sizeCache.width(), sizeCache.height());
		} else {
			var dw = 120;
			if(!editMenuIsOpen) {
				dw = -dw;
			}
			var h = NB.getHostObject();
			var newWidth = h.width + dw;
			var mins = self.minSizes();
			if(newWidth < mins.width) {
				newWidth = mins.width;
			}
			var ratio = editMenuIsOpen ? 0.8 : 1;
			newHeight = newWidth * ratio;
			self.setAnnSize(newWidth, newHeight);
		}
	};

	self.setAnnSize = function(w, h) {
		var host = NB.getHostObject();
		NB.undomanager.setAutoRecord(false);
		host.width = w;
		host.height = h;
		NB.undomanager.setAutoRecord(true);
	};


	//Themes
	self.setTheme = function(themeName) {
		self._setTheme(diceThemer.getThemeOption(themeName));
		self.setFolded(false);
		self.layout();
		saveData();
	};

	self._setTheme = function(_theme) {
		theme = _theme;
		updateDiceTheme();
		$("#traditional-theme, #letters-theme, #custom-theme").css('background-color','#00BFDD');
		$("#" + _theme.type + "-theme").css('background-color','#1a75ff');
		updateInfoText();
	};

	function setDiceTheme(_dice, theme) {
		for(var i = 0; i < 6; i++) {
			_dice.changeFaceContent(i, theme.content[i]);
		}
	}

	function updateDiceTheme() {
		for(var i = 0; i < dice.length; i++) {
			setDiceTheme(dice[i], theme);
		}
		self.setDiceRgb(currentRgb);
		resizeAllDiceText();
		self.setTextEnabled(editMenuIsOpen);
		saveData();
	}

	//Other
	// When notified by nbImageSniffer that an image has been dropped within the widget, checks
	// if it was dropped on a dice face, and adds it to the dice if so.
	function onImageSniffed(data) {
		if(!dice[0].isFolded() && theme.type === "Custom") {
			var imageHtml = diceThemer.getImageHTML(data.img);
			var element = document.elementFromPoint(data.x, data.y);
			while(element !== null) {
				if(element.className && element.className.indexOf("content") >= 0) {
					var className = element.parentNode.className;
					var index = Number(className[4]) - 1;
					doImageDrop(data, index, theme.content[index], imageHtml);
					break;
				}
				element = element.parentNode;
			}
		}
		saveData();
	}

	function doImageDrop(data, index, oldHtml, newHtml) {
		NB.undomanager.startRecord();
		NB.undomanager.performUndoableAction(
			function() {
				diceThemer.getThemeOption("Custom").content[index] = newHtml;
				updateDiceTheme();
			},
			function() {
				diceThemer.getThemeOption("Custom").content[index] = oldHtml;
				updateDiceTheme();
			}
		);
		data.annotation.deleteObject();
		NB.undomanager.stopRecord();
	}

	self.setTextEnabled = function(value) {
		var sel = value ? "text" : "none";
		var elements = $(".letter");
		elements.css({
		   "-moz-user-select": sel,
		   "-khtml-user-select": sel,
		    "-webkit-user-select": sel,
		    "-o-user-select": sel,
		});
		elements.attr("contenteditable", value);
	};

	self.onDiceClick = function() {
		if(!editMenuIsOpen) {
			spinAllDice();
		}	
	};

	self.onSettingsClick = function() {
		if(!diceSpin) {
			self.toggleSettings();
		}
	};

	self.onWindowResize = function() {
		sizeCache.onResize();
		var win = $(window);
		var showSettings = win.width() > win.height() + 10;
		self._setEditMenuIsOpen(showSettings, !showSettings);
		self.updateMinSizes();
		saveData();
		self.layout();
		setTimeout(self.layout, 10); //Sometimes sent improper coords, so relayout later on just in case.
	};

	self.updateTextEnabledDelayed = function() {
		setTimeout(function() {
			self.setTextEnabled(!dice[0].isFolded());
		}, 100);
	};

	self.updatePlusMinus = function() {
		var n = dice.length;
		$("#controls-add").css({
			opacity: n === 9 ? 0.6 : 1,
		});
		$("#controls-remove").css({
			opacity: n === 1 ? 0.6 : 1,
		});
	};

	self.updateMinSizes = function() {
		var mins = self.minSizes();
		NB.getHostObject().setWidgetMinSize(mins.width, mins.height);
	};

	self.minSizes = function() {
		return {
			width: editMenuIsOpen ? 324 : 160,
			height: 1,
		};
	};

	self.minSizesFor = function(editIsOpen) {
		return {
			width: editIsOpen ? 324 : 160,
			height: 1,
		};
	};

	self.updatePreview = function(delay) {
		if(!delay) {
			delay = 0;
		}
		setTimeout(function() {
			NB.page.updateThumbnail();
		}, delay);
	};

	self.onFocusout = function() {
		if(!self.selectionListener.isSelected()) {
			self.settingsButton.setShownAnim(false);
		}
	};

	self.onSelectionChange = function(listener, isSelected) {
		if(isSelected) {
			if(!diceSpin) {
				self.settingsButton.setShownAnim(true);
			}
		} else {
			self.settingsButton.setShownAnim(false);
		}
	};

	init();
}

