function getIdentityMatrix() {
    var matrix = [
        1, 0, 0, 0,
        0, 1, 0, 0,
        0, 0, 1, 0,
        0, 0, 0, 1];
    return matrix;
}

function getTranslationMatrix(x, y, z) {
    var matrix = [
        1, 0, 0, 0,
        0, 1, 0, 0,
        0, 0, 1, 0,
        x, y, z, 1];
    return matrix;
}

function getRotationMatrixX(angle) {
    angle = toRadians(angle);
    var matrix = [
        1, 0, 0, 0,
        0, Math.cos(angle), -Math.sin(angle), 0,
        0, Math.sin(angle), Math.cos(angle), 0,
        0, 0, 0, 1];
    return matrix;
}

function getRotationMatrixY(angle) {
    angle = toRadians(angle);
    var matrix = [
        Math.cos(angle), 0, Math.sin(angle), 0,
        0, 1, 0, 0,
        -Math.sin(angle), 0, Math.cos(angle), 0,
        0, 0, 0, 1];
    return matrix;
}

function getRotationMatrixZ(angle) {
    angle = toRadians(angle);
    var matrix = [
        Math.cos(angle), -Math.sin(angle), 0, 0,
        Math.sin(angle), Math.cos(angle), 0, 0,
        0, 0, 1, 0,
        0, 0, 0, 1];
    return matrix;
}

function getScalingMatrix(x, y, z) {
    var matrix = [
        x, 0, 0, 0,
        0, y, 0, 0,
        0, 0, z, 0,
        0, 0, 0, 1];
    return matrix;
}

function multiplyMatrix(m1, m2) {
    var matrix = [];

    matrix[0]   = m1[0]*m2[0]+m1[1]*m2[4]+m1[2]*m2[8]+m1[3]*m2[12];
    matrix[1]   = m1[0]*m2[1]+m1[1]*m2[5]+m1[2]*m2[9]+m1[3]*m2[13];
    matrix[2]   = m1[0]*m2[2]+m1[1]*m2[6]+m1[2]*m2[10]+m1[3]*m2[14];
    matrix[3]   = m1[0]*m2[3]+m1[1]*m2[7]+m1[2]*m2[11]+m1[3]*m2[15];
    matrix[4]   = m1[4]*m2[0]+m1[5]*m2[4]+m1[6]*m2[8]+m1[7]*m2[12];
    matrix[5]   = m1[4]*m2[1]+m1[5]*m2[5]+m1[6]*m2[9]+m1[7]*m2[13];
    matrix[6]   = m1[4]*m2[2]+m1[5]*m2[6]+m1[6]*m2[10]+m1[7]*m2[14];
    matrix[7]   = m1[4]*m2[3]+m1[5]*m2[7]+m1[6]*m2[11]+m1[7]*m2[15];
    matrix[8]   = m1[8]*m2[0]+m1[9]*m2[4]+m1[10]*m2[8]+m1[11]*m2[12];
    matrix[9]   = m1[8]*m2[1]+m1[9]*m2[5]+m1[10]*m2[9]+m1[11]*m2[13];
    matrix[10]  = m1[8]*m2[2]+m1[9]*m2[6]+m1[10]*m2[10]+m1[11]*m2[14];
    matrix[11]  = m1[8]*m2[3]+m1[9]*m2[7]+m1[10]*m2[11]+m1[11]*m2[15];
    matrix[12]  = m1[12]*m2[0]+m1[13]*m2[4]+m1[14]*m2[8]+m1[15]*m2[12];
    matrix[13]  = m1[12]*m2[1]+m1[13]*m2[5]+m1[14]*m2[9]+m1[15]*m2[13];
    matrix[14]  = m1[12]*m2[2]+m1[13]*m2[6]+m1[14]*m2[10]+m1[15]*m2[14];
    matrix[15]  = m1[12]*m2[3]+m1[13]*m2[7]+m1[14]*m2[11]+m1[15]*m2[15];

    return matrix;
}

function multiplyVectorMatrix(v1, m1) {
    var vector = [
        v1[0] * m1[0] + v1[1] * m1[4] + v1[2] * m1[8] + 1 * m1[12],
        v1[0] * m1[1] + v1[1] * m1[5] + v1[2] * m1[9] + 1 * m1[13],
        v1[0] * m1[2] + v1[1] * m1[6] + v1[2] * m1[10] + 1 * m1[14],
        v1[0] * m1[3] + v1[1] * m1[7] + v1[2] * m1[11] + 1 * m1[15]
    ];

    return vector;
}

function multiplyVectorMatrix2(v1, m1) {
    var vector = [
        v1[0] * m1[0] + v1[1] * m1[1] + v1[2] * m1[2] + v1[3] * m1[3],
        v1[0] * m1[4] + v1[1] * m1[5] + v1[2] * m1[6] + v1[3] * m1[7],
        v1[0] * m1[8] + v1[1] * m1[9] + v1[2] * m1[10] + v1[3] * m1[11],
        v1[0] * m1[12] + v1[1] * m1[13] + v1[2] * m1[14] + v1[3] * m1[15]
    ];

    return vector;
}

function crossProduct(v1, v2) {
    var vector = [
        v1[1] * v2[2] - v1[2] * v2[1],
        v1[2] * v2[0] - v1[0] * v2[2],
        v1[0] * v2[1] - v1[1] * v2[0],
    ];

    return vector;
}

function toRadians(angle) {
    return angle / 180 * Math.PI;
}

function toDegrees(angle) {
    return angle * 180 / Math.PI;
}

function dotProduct(v1, v2) {
    return v1[0] * v2[0] + v1[1] * v2[1] + v1[2] * v2[2];
}

function getMagnitude(v) {
    return Math.sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
}

// Calculates the axis and angle necessary to align v1 to v2
// @params v1 - the vector to rotate
// @params v2 - the vector to rotate to
// @params rotationAxis - the axis of rotation used when an axis cannot be found.
//      i.e. when v1 and v2 are parallel
function alignVectors(v1, v2, rotationAxis) {
    var u = normalize(v1);
    var v = normalize(v2);
    var _axis, _angle;
    if (getMagnitude(vectorSubtract(u, v)) < 0.001) {
        _axis = [0, 0, 1];
        _angle = 0;
    } else {
        _axis = normalize(crossProduct(u, v));
        _angle = Math.acos(dotProduct(u, v));
    }
    // If the alignTo vector is opposite the orientationVector,
    // find a random perpendicular vector and rotate 180 degrees around that
    // Potential Bug: If we're trying to correct it's upright position,
    // then we might mess up the rotation again.
    // Fix: Pass in the axis to rotate on
    if (isNaN(_axis[0])) {
        if (rotationAxis) {
            _axis = rotationAxis;
        } else {
            var x = getRandomArbitrary(0, 1);
            var y = getRandomArbitrary(0, 1);
            z = -(x * u[0] + y * u[1]) / u[2];
            _axis = normalize([x, y, z]);
        }
        _angle = Math.PI;
    }
    return {axis: _axis, angle: _angle};
}

function alignVectorsOnZ(v1, v2) {
    v1[2] = v2[2] = 0;
    var u = normalize(v1);
    var v = normalize(v2);
    var _axis, _angle;
    if (getMagnitude(vectorSubtract(u, v)) < 0.001) {
        _axis = [0, 0, 1];
        _angle = 0;
    } else {
        _axis = normalize(crossProduct(u, v));
        _angle = Math.acos(dotProduct(u, v));
    }
    if (isNaN(_axis[0])) {
        _axis = [0, 0, 1];
        _angle = Math.PI;
    }
    return {axis: _axis, angle: _angle};
}

function normalize(v) {
    var m = getMagnitude(v);
    var vector = [
        v[0] / m,
        v[1] / m,
        v[2] / m];
    return vector;
}

function vectorSubtract(v1, v2) {
    var vector = [
        v1[0] - v2[0],
        v1[1] - v2[1],
        v1[2] - v2[2]
    ];
    return vector;
}

// Returns a random number from min (inclusive), to max (exclusive)
function getRandomArbitrary(min, max) {
    return Math.random() * (max - min) + min;
}

// Returns a random number -max to -min, or min to max
function getRandomArbitraryFudge(min, max, fudge) {
    var rand = getRandomArbitrary(min + fudge, max - fudge);
    if(rand < 0) {
        return rand - fudge;
    } else {
        return rand + fudge;
    }
}

// Returns a random number from min (inclusive), to max (inclusive)
function getRandomInt(min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
}