/* Object which easily allows us to keep track of a previously set size, when user is toggling in and out of settings.
	Scenario 1:
		- User has dice set to smallest possible size with settings hidden
		- User clicks settings (resizing widget to larger value to fit everything)
		- User closes settings
		- Now we want to go back to the previous small size.

	Scenario 2:
		- User has dice set to smallest possible size with settings hidden
		- User clicks settings (resizing widget to larger value to fit everything)
		- User resizes widget to different size
		- User closes settings
		- Now we don't want to previous small size b.c user has explicitly set to new size.

	Notes
		- Since NBSDK currently pipes resize events in a semi unpredictable behavior, it can be hard to tell whether a resize
		was triggered by a user or programmitcally.
		- To keep thing simple, any resize event sent through NBSDK after a certain period of time beyond the settings click
		will be assumed to be a manual resize. (And thus invalidate the cache)

	Vars
		isValid : Boolean -- true if cache is still valid
		cacheTime : Number -- time at which time was clicked. milliseconds
		delay : Number -- Amount of time to wait before assuming resize events are maually triggered. milliseconds
*/

function SizeCache() {
	
	//Vars
	var self = this;
	var width = 0;
	var height = 0;
	var isValid = false;
	var cacheTime = 0;
	var delay = 100;

	//Functions
	self.isValid = function() {
		return isValid;
	};

	self.width = function() {
		return width;
	};

	self.height = function() {
		return height;
	};

	self.cache = function() {
		isValid = true;
		cacheTime = Date.now();
		var host = NB.getHostObject();
		width = host.width;
		height = host.height;
	};

	self.onResize = function() {
		if(Date.now() - cacheTime > delay) {
			isValid = false;
		}
	};
};
